/*
 *
 * HGA v1.3.1
 * Program: makepop, which generates input files for the HGA
 * File: error.c, contains error functions
 *
 * Copyright (c) 1997, 1995, 1994 Stephen D. Scott and the University of
 * Nebraska-Lincoln.  All rights reserved.
 *
 * Permission to use or modify this software and its documentation only
 * for educational purposes and without fee is hereby granted, provided
 * that the above copyright notice appears in all copies.  The author
 * and university make no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.  The author and university shall not be liable in
 * any event for incidental or consequential damages in connection
 * with, or arising out of, the furnishing, performance, or use of
 * these programs.
 *
 */

#include "makepop.h"

/* 
 * checks command line arguments 
 * available args: -p, -n, -f, -ca, -m, -g, -mutpr, -xpr, -valw, -memsize,
 *                 -out, -help
 * defaults exist for all args
 */
void CheckArgs(int argc, char *argv[])
{
int i;
char string[100];

for (i=1;i<argc;i++)
  {
    if (strcmp(argv[i], "-p") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &p) == EOF) || (p < 1))
          Error("Invalid value for p");
      }
    else if (strcmp(argv[i], "-n") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &n) == EOF) || (n < 1))
          Error("Invalid value for n");
      }
    else if (strcmp(argv[i], "-f") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &f) == EOF) || (f < 1))
          Error("Invalid value for f");
      }
    else if (strcmp(argv[i], "-m") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &m) == EOF) || (m < 1))
          Error("Invalid value for m");
      }
    else if (strcmp(argv[i], "-g") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &g) == EOF) || (g < 1))
          Error("Invalid value for g");
      }
    else if (strcmp(argv[i], "-mutpr") == 0)
      {
        mutprob = strtod(argv[++i],NULL);
        if ((mutprob < 0.0) || (mutprob > 1.0))
          Error("Invalid value for mutprob");
      }
    else if (strcmp(argv[i], "-xpr") == 0)
      {
        xovprob = strtod(argv[++i],NULL);
        if ((xovprob < 0.0) || (xovprob > 1.0))
          Error("Invalid value for xovprob");
      }
    else if (strcmp(argv[i], "-valw") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &valw) == EOF) || (valw < 1))
          Error("Invalid value for valw");
      }
    else if (strcmp(argv[i], "-memsize") == 0) 
      {
        if ((sscanf(argv[++i], "%d", &memsize) == EOF) || (memsize < 1))
          Error("Invalid value for memsize");
      }
    else if (strcmp(argv[i], "-out") == 0) 
      {
        if ((out = fopen(fname = argv[++i], "w")) == NULL)
          {
            sprintf(string, "Could not open file %s", argv[i]);
            Error(string);
          }
      }
    else if (strcmp(argv[i], "-seed") == 0) 
      {
        if (sscanf(argv[++i], "%d", &oldseed) == EOF)
          Error("Invalid value for seed");
        useseed=1;
      }
    else if (strcmp(argv[i], "-help") == 0) 
      {
        Usage(NULL, argv[0]);
      }
    else Usage(argv[i], argv[0]);
  }
}

/*
 * prints usage message and exits
 */
void Usage(char *invalidarg, char *name)
{
if (invalidarg != NULL)
  fprintf(stderr, "Illigal option: %s\n", invalidarg);
fprintf(stderr, "Usage: %s [-p <prob. size>] [-n <member width>] ", name);
fprintf(stderr, "[-f <fitness width>]\n");
fprintf(stderr, "               [-m <pop. size>] [-g <num. gens>] ");
fprintf(stderr, "[-valw <word width>]\n");
fprintf(stderr, "               [-memsize <no. words>] ");
fprintf(stderr, "[-mutpr <mut. prob.>] [-xpr <xov. prob.>]\n");
fprintf(stderr, "               -out <file> [-help]\n");
fprintf(stderr, "NOTE: Specify parameters by their true values.  The generated");
fprintf(stderr, " file will\n      hold the adjusted parameters.\n");
exit(-1);
}


/*
 * prints error message and exits
 */
void Error(char *message)
{
fprintf(stderr, "Error: %s\n", message);
fflush(NULL);
exit(-1);
}
